PRODUCER_KEY = 'producer'
PRODUCER_NAME = 'ymir'
EXECUTOR_CONFIG_KEY = 'executor_config'
TASK_CONTEXT_KEY = 'task_context'
TASK_CONTEXT_PARAMETERS_KEY = 'task_parameters'
EXECUTOR_OUTLOG_NAME = 'ymir-executor-out.log'
